/* random.cpp
   Modified filename from "random.c" to "random.cpp"
   @author:		Hyun Kim
   @date:		11/20/99
*/
/* -*- c -*- */
/*
 *  File        : random.cpp
 *  Authors     : Fernando Lobo, Georges Harik, David Goldberg
 *  Version     : LLGA version 1.0
 *  Date        : Apr/1998
 *
 *  Description : C implementation of random number generator.
 *                Translated from Goldberg's SGA code in Pascal.
 *
 */

#include <stdio.h>
#include "random.hpp"

double  oldrand[55];    /* array of 55 random numbers */
int     jrand;          /* current random */

/* forward declarations */
void warmup_random( const double seed );
void advance_random();

/*
 *  Initialize random number generator
 */
void randomize( double seed ) 
{
  warmup_random( seed );
}

/* 
 *  Get random off and runnin 
 */
void warmup_random( const double seed )
{
  int i,j;
  double new_random, prev_random;
 
  oldrand[54] = seed;
  new_random = 1.0e-9;
  prev_random = seed;
  for( j=1; j<=54; j++ )
    {
      i = 21*j % 54;
      oldrand[i] = new_random;
      new_random = prev_random - new_random;
      if (new_random < 0.0) new_random++;
      prev_random = oldrand[i];
    }
  advance_random(); advance_random(); advance_random();
  jrand = 0;
}

/* 
 *  Create next batch of 55 random numbers 
 */
void advance_random()
{
  int    i;
  double new_random;

  for( i=0; i<24; i++ )
    {
      new_random = oldrand[i] - oldrand[i+31];
      if (new_random < 0.0) new_random++;
      oldrand[i] = new_random;
    }
  for( i=24; i<55; i++ )
    {
      new_random = oldrand[i] - oldrand[i-24];
      if (new_random < 0.0) new_random++;
      oldrand[i] = new_random;
    }
}

/*
 *  Fetch a single random number between 0.0 and 1.0 - Subtractive Method
 *  See Knuth, D. (1969), v. 2 for details
 */
double random01()
{
  jrand++;
  if (jrand>=55)
    {
      jrand = 1;
      advance_random();
    }
  return oldrand[jrand];
}

/* 
 *  Flip a biased coin and returns 0 or 1 
 */
int flip( double probability )
{
  if (probability == 1.0) return 1;
  return random01() <= probability;
}

/*
 *  Pick a random integer in [low..high]
 */
int rnd( int low, int high )
{
  int i;

  if (low >= high) i=low;
  else
    {
      i = (int) (random01() * (high-low+1) + low);
      if (i>high) i=high;
    }
  return i;
}
